/*
 * Copyright 2017 aquenos GmbH.
 * All rights reserved.
 * 
 * Please refer to the file NOTICE.txt in the root of the source distribution or
 * in the META-INF directory of the binary archive for licensing information.
 */

package com.aquenos.epics.jackie.diirt.datasource.internal;

import org.diirt.vtype.VDouble;

import com.aquenos.epics.jackie.common.value.ChannelAccessControlsDouble;
import com.aquenos.epics.jackie.common.value.ChannelAccessTimeDouble;
import com.aquenos.epics.jackie.common.value.ChannelAccessValueType;
import com.aquenos.epics.jackie.diirt.vtype.JackieVDouble;

/**
 * Type adapter that convert from {@link ChannelAccessControlsDouble} and
 * {@link ChannelAccessTimeDouble} to {@link VDouble}. This adapter can only be
 * used if the value has exactly one element.
 * 
 * @author Sebastian Marsching
 */
public class ScalarDoubleTypeAdapter
        extends JackieDataSourceTypeAdapter<VDouble> {

    /**
     * Single instance of this adapter.
     */
    public static final ScalarDoubleTypeAdapter INSTANCE = new ScalarDoubleTypeAdapter();

    private ScalarDoubleTypeAdapter() {
        super(true, ChannelAccessValueType.DBR_DOUBLE, VDouble.class);
    }

    @Override
    protected VDouble convertValue(JackieConnectionPayload connection,
            JackieMessagePayload message) {
        // We can safely make a cast here because this adapter is only used if
        // the channel's native type is DBR_DOUBLE and we thus know that the
        // monitors return a DBR_CTRL_DOUBLE and DBR_TIME_DOUBLE.
        ChannelAccessControlsDouble controlsValue = (ChannelAccessControlsDouble) message
                .getControlsValue();
        ChannelAccessTimeDouble timeValue = (ChannelAccessTimeDouble) message
                .getTimeValue();
        return new JackieVDouble(controlsValue, timeValue,
                connection.getDisconnectTime(),
                connection.isHonorZeroPrecision());
    }

}
